#ifndef CHESSY_TRANSPOSITIONTABLE_H
#define CHESSY_TRANSPOSITIONTABLE_H

#include <array>
#include <cinttypes>
#include <unordered_map>

#include "Evaluate.h"
#include "Bitboard.h"

namespace chessy
{
    using HashValue = uint64_t;

    struct PositionInfo;
    class TranspositionTable;

    // forward declaration
    class ChessGame;

    /*!
     * \brief namespace containing the constant values used for zobrist hashing.
     */
    namespace zobrist
    {
        extern std::array<HashValue, 64 * 12> pieces;
        extern std::array<HashValue, 16> castlingRights;
        extern std::array<HashValue, 8> enPassant;
        extern HashValue turn;

        HashValue getHash(const ChessGame&);

        inline HashValue get(PieceType p, Side side, Index i)
        {
            return pieces[(static_cast<int>(p) + side * 6) * 64 + i];
        }

        void initialize(void);
    }
}


struct chessy::PositionInfo
{
    int evaluatedDepth;
    MoveValue value;
    Move bestMove;
};


class chessy::TranspositionTable
{
    std::unordered_map<HashValue, PositionInfo> table;
};

#endif //CHESSY_TRANSPOSITIONTABLE_H

